﻿using ELTE.TravelAgency.Data;
using ELTE.TravelAgency.Service.Models;
using System;
using System.Linq;
using System.Web.Http;

namespace ELTE.TravelAgency.Service.Controllers
{
    /// <summary>
    /// Épületek képeinek kezelését biztosító vezérlő.
    /// </summary>
    [RoutePrefix("api/images")]
    public class BuildingImagesController : ApiController
    {
        private ITravelAgencyEntities _entities;

        /// <summary>
        /// Vezérlő példányosítása.
        /// </summary>
        public BuildingImagesController()
            : this(new TravelAgencyEntities())
        {
        }
        
        /// <summary>
        /// Vezérlő példányosítása.
        /// </summary>
        /// <param name="entities">Entitásmodell.</param>
        public BuildingImagesController(ITravelAgencyEntities entities)
        {
            if (entities == null)
                throw new ArgumentNullException("entities");

            _entities = entities;
        }
        
        /// <summary>
        /// Vezérlő megsemmisítése.
        /// </summary>
        ~BuildingImagesController()
        {
            Dispose(false);
        }

        /// <summary>
        /// Egy épület képeinek lekérdezése.
        /// </summary>
        /// <param name="buildingId">Épület azonosító.</param>
        [Route("building/{buildingId}")]
        public IHttpActionResult GetImages(Int32 buildingId)
        {
            // a megfelelő képeket betöltjük és átalakítjuk (csak a kis képeket)
            return Ok(_entities.BuildingImage.Where(image => image.BuildingId == buildingId).Select(image => new ImageDTO { Id = image.Id, ImageSmall = image.ImageSmall }));
        }

        /// <summary>
        /// Egy kép lekérdezése.
        /// </summary>
        [Route("{id}")]
        public IHttpActionResult GetImage(Int32 id)
        {
            BuildingImage image = _entities.BuildingImage.FirstOrDefault(img => img.BuildingId == id);

            if (image == null)
                return NotFound();

            // a képe átalakítjuk (kis és nagyképet egyaránt)
            return Ok(new ImageDTO 
            { 
                Id = image.Id,
                BuildingId = image.BuildingId,
                ImageSmall = image.ImageSmall, 
                ImageLarge = image.ImageLarge
            });
        }

        /// <summary>
        /// Kép feltöltése.
        /// </summary>
        /// <param name="buildingId">Épület azonosító.</param>
        /// <param name="size">Képméret.</param>
        /// <param name="content">Képi tartalom.</param>
        [Route] // itt nem kell paramétereznünk, csak jelezzük, hogy az egyedi útvonalat vesszük igénybe
        [Authorize(Roles = "administrator")] // csak bejelentkezett adminisztrátoroknak
        public IHttpActionResult PostImage([FromBody] ImageDTO image)
        {
            if (image == null || !_entities.Building.Any(building => image.BuildingId == building.Id))
                return NotFound();

            BuildingImage buildingImage = new BuildingImage
            {
                BuildingId = image.BuildingId,
                ImageSmall = image.ImageSmall,
                ImageLarge = image.ImageLarge
            };

            _entities.BuildingImage.Add(buildingImage);

            try
            {
                _entities.SaveChanges();
                return Created(Request.RequestUri + image.Id.ToString(), image.Id); // csak az azonosítót küldjük vissza
            }
            catch
            {
                return InternalServerError();
            }
        }

        /// <summary>
        /// Kép törlése.
        /// </summary>
        /// <param name="id">A kép azonosítója.</param>
        [Route("{id}")]
        [Authorize(Roles = "administrator")]
        public IHttpActionResult DeleteImage(Int32 id)
        {
            BuildingImage image = _entities.BuildingImage.FirstOrDefault(im => im.BuildingId == id);

            if (image == null)
                return NotFound();

            try
            {
                _entities.BuildingImage.Remove(image);
                _entities.SaveChanges();
                return Ok();
            }
            catch
            {
                return InternalServerError();
            }
        }

        /// <summary>
        /// Megsemmisítés.
        /// </summary>
        protected override void Dispose(Boolean disposing)
        {
            if (disposing)
            {
                _entities.Dispose();
            }

            base.Dispose(disposing);
        }
    }
}
